<?php
namespace Datec\DatecJobmarket\Services;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use Datec\DatecJobmarket\Domain\Model\FacilityType;
use Datec\DatecJobmarket\Domain\Model\Jobentry;
use Datec\DatecJobmarket\Domain\Model\Jobgroup;
use Datec\DatecJobmarket\Domain\Model\Qualification;
use Datec\DatecJobmarket\Domain\Model\WorktimeType;
use TYPO3\CMS\Core\Mail\MailMessage;
use TYPO3\CMS\Core\SingletonInterface;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
use TYPO3\CMS\Fluid\View\StandaloneView;

/**
 * @package datec_jobmarket
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class MailService implements SingletonInterface  {
	/**
	 * $configurationManager
	 * @var ConfigurationManagerInterface
	 * */
	protected $configurationManager;

    /**
     * @param ConfigurationManagerInterface $configurationManager
     */
    public function injectConfigurationManager(ConfigurationManagerInterface $configurationManager)
    {
        $this->configurationManager = $configurationManager;
    }

    /**
     * Sends mail with message to an intern recipients.
     *
     * @param string $subject Subject of message
     * @param string $msg Message text
     * @param array $settings Plugin configuration
     *
     * @return boolean Message was send
     */
	public function sendInternMail($subject, $msg, $settings) {		
		$message = new MailMessage();
		$message->setFrom(array($settings['mail']['internMailFrom'] => $settings['mail']['internMailFromName']));
		$message->setTo(array($settings['mail']['internMailTo'] => ''));
		if ($settings['mail']['sendSupportInternMail'] == 1) {
			$message->setBcc(array($settings['mail']['supportInternMailTo'] => ''));
		}
		$message->setSubject($subject);
		$message->setBody($msg,'text/html', 'utf-8');
		
		$message->send();
		if($message->isSent()) {
			return TRUE;
		}
	
		return FALSE;
	}

    /**
     * send informative mail to intern recipients
     *
     * @param string $msg
     * @param array $settings
     * @param string $extKey
     * @return boolean $sendMail success if message is sent
     */
	public function sendInfoMail($msg, $settings, $extKey){
		$subject = LocalizationUtility::translate('tx_datecjobmarket.mail.subjects.infoMail', $extKey);
		$sendMail = MailService::sendInternMail($subject, $msg, $settings);
	
		return $sendMail;
	}

    /**
     * Generate the HTML Info E-Mail
     * @param Jobentry $jobentry
     * @param string $extKey
     * @param $extensionName
     * @return string
     */
	public function generateInfoMail($jobentry, $extKey, $extensionName) {
		$extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

        $mailView = new StandaloneView();
        $mailView->setTemplateRootPaths($extbaseFrameworkConfiguration['view']['templateRootPaths']);
        $mailView->setTemplate('/Email/InfoMail.html');
        $mailView->getRequest()->setControllerExtensionName($extensionName);
		
		$properties = $jobentry->_getCleanProperties();
		$propertiesArr = array();
		foreach ($properties as $propertyName => $propertyValue) {
			if (!empty($propertyValue) && $propertyName != 'uid' && $propertyName != 'pid' && $propertyName != 'hidden' && $propertyName != 'starttime' && $propertyName != 'endtime') { // only non-empty, non-system properties

				$valueStr = "";
				if ($propertyValue instanceof ObjectStorage) {
				    /* @var Jobgroup|Qualification $object */
					foreach ($propertyValue->toArray() as $object) {
						$valueStr .= LF.$object->getName();
					}
					$valueStr .= LF;
				} else if ($propertyValue instanceof FacilityType || $propertyValue instanceof WorktimeType) {
					$valueStr = $propertyValue->getName();
				} else if ($propertyValue instanceof \DateTime) {
					$valueStr = $propertyValue->format($extbaseFrameworkConfiguration['settings']['display']['dateFormat']);
				} else if ($propertyName == 'jobtype') {
					$valueStr = LocalizationUtility::translate('tx_datecjobmarket.jobentry.jobtype.'.$propertyValue,$extKey);
				} else if ($propertyName == 'exportable') {
					$valueStr = LocalizationUtility::translate('tx_datecjobmarket.jobentry.exportable.'.$propertyValue,$extKey);
                } else if ($propertyName == 'anonymous') {
                    $valueStr = LocalizationUtility::translate('tx_datecjobmarket.jobentry.anonymous.'.$propertyValue,$extKey);
				}	else if ($propertyName == 'password') {
					$valueStr = LocalizationUtility::translate('tx_datecjobmarket.messages.protected',$extKey);
				} else {
					$valueStr = $propertyValue;
				}
				$propertiesArr['properties'][] = array(
						"name" => LocalizationUtility::translate('tx_datecjobmarket.jobentry.'.$propertyName, $extKey),
						"value" => $valueStr,
				);
			}
		}
		
		$mailView->assignMultiple($propertiesArr);
		$emailBody = $mailView->render();
		
		return $emailBody;
	}

    /**
     * Send mail to contact Jobentry creator
     *
     * @param string $msg
     * @param Jobentry $jobentry
     * @param string $subject
     * @param $settings
     * @return boolean $sendMail success if message is sent
     */
	public function sendContactMail($msg, $jobentry, $subject, $settings) {
		$mail = new MailMessage();
		$mail->setFrom(array($settings['mail']['contactMailFromName'] => $settings['mail']['contactMailFrom']));
		$mail->setTo($jobentry->getEmail(), '');
		
		$mail->setBody($msg, 'text/html', 'utf-8');
		$mail->setSubject($subject);
		
		$mail->send();
		if($mail->isSent()) {
			return true;
		}

		return false;
	}

    /**
     * Generate a HTML template for contact mail
     *
     * @param $contactAddress
     * @param $contactMsg
     * @param $extensionName
     * @return string
     */
	public function generateContactMail($contactAddress, $contactMsg, $extensionName) {
		$extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

        $mailView = new StandaloneView();
		$mailView->setTemplateRootPaths($extbaseFrameworkConfiguration['view']['templateRootPaths']);
		$mailView->setTemplate('/Email/ContactMail.html');
		$mailView->getRequest()->setControllerExtensionName($extensionName);
		
		$contactArr =  [
		    'contactAddress' => $contactAddress,
            'contactMsg' => $contactMsg
        ];
		
		$mailView->assignMultiple($contactArr);
		$emailBody = $mailView->render();

		return $emailBody;
	}
}